// Vikunja is a to-do list application to facilitate your life.
// Copyright 2018-2021 Vikunja and contributors. All rights reserved.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public Licensee as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public Licensee for more details.
//
// You should have received a copy of the GNU Affero General Public Licensee
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

package models

import (
	"code.vikunja.io/api/pkg/events"
	"code.vikunja.io/api/pkg/metrics"
	"github.com/ThreeDotsLabs/watermill/message"
)

func RegisterListeners() {
	events.RegisterListener((&ListCreatedEvent{}).TopicName(), &IncreaseListCounter{})
	events.RegisterListener((&ListDeletedEvent{}).TopicName(), &DecreaseListCounter{})
	events.RegisterListener((&NamespaceCreatedEvent{}).TopicName(), &IncreaseNamespaceCounter{})
	events.RegisterListener((&NamespaceDeletedEvent{}).TopicName(), &DecreaseNamespaceCounter{})
	events.RegisterListener((&TaskCreatedEvent{}).TopicName(), &IncreaseTaskCounter{})
	events.RegisterListener((&TaskDeletedEvent{}).TopicName(), &DecreaseTaskCounter{})
}

//////
// Task Events

// IncreaseTaskCounter  represents a listener
type IncreaseTaskCounter struct {
}

// Name defines the name for the IncreaseTaskCounter listener
func (s *IncreaseTaskCounter) Name() string {
	return "increase.task.counter"
}

// Hanlde is executed when the event IncreaseTaskCounter listens on is fired
func (s *IncreaseTaskCounter) Handle(payload message.Payload) (err error) {
	metrics.UpdateCount(1, metrics.TaskCountKey)
	return nil
}

// DecreaseTaskCounter  represents a listener
type DecreaseTaskCounter struct {
}

// Name defines the name for the DecreaseTaskCounter listener
func (s *DecreaseTaskCounter) Name() string {
	return "decrease.task.counter"
}

// Hanlde is executed when the event DecreaseTaskCounter listens on is fired
func (s *DecreaseTaskCounter) Handle(payload message.Payload) (err error) {
	metrics.UpdateCount(-1, metrics.TaskCountKey)
	return nil
}

///////
// List Event Listeners

type IncreaseListCounter struct {
}

func (s *IncreaseListCounter) Name() string {
	return "list.counter.increase"
}

func (s *IncreaseListCounter) Handle(payload message.Payload) (err error) {
	metrics.UpdateCount(1, metrics.ListCountKey)
	return nil
}

type DecreaseListCounter struct {
}

func (s *DecreaseListCounter) Name() string {
	return "list.counter.decrease"
}

func (s *DecreaseListCounter) Handle(payload message.Payload) (err error) {
	metrics.UpdateCount(-1, metrics.ListCountKey)
	return nil
}

//////
// Namespace events

// IncreaseNamespaceCounter  represents a listener
type IncreaseNamespaceCounter struct {
}

// Name defines the name for the IncreaseNamespaceCounter listener
func (s *IncreaseNamespaceCounter) Name() string {
	return "namespace.counter.increase"
}

// Hanlde is executed when the event IncreaseNamespaceCounter listens on is fired
func (s *IncreaseNamespaceCounter) Handle(payload message.Payload) (err error) {
	metrics.UpdateCount(1, metrics.NamespaceCountKey)
	return nil
}

// DecreaseNamespaceCounter  represents a listener
type DecreaseNamespaceCounter struct {
}

// Name defines the name for the DecreaseNamespaceCounter listener
func (s *DecreaseNamespaceCounter) Name() string {
	return "namespace.counter.decrease"
}

// Hanlde is executed when the event DecreaseNamespaceCounter listens on is fired
func (s *DecreaseNamespaceCounter) Handle(payload message.Payload) (err error) {
	metrics.UpdateCount(-1, metrics.NamespaceCountKey)
	return nil
}
